clear all
close all
format short e

% In order to replicate the experiments described in the paper you need to
% set only a few parameters: simulation, nexp, bias, c_error See 
% documentation for details.

% true for simulations, false for experiments with the real robot

simulation = true;

% nexp is the number of the experiment to be performed; it is needed to
% load the appropriate data files
nexp = 2;

% bias: positive multiplicative factor for the initial guess by the 
% proposed  estimator;

bias = 100;

% c_error: calibration error, i.e. the error in covarince values for
% EKF and IKF

c_error = 3;

%% Real processing starts here

% conversion factors
inchesToMeters = 0.0254;
feetToMeters = 0.3048;

options=optimset;
options.MaxIter= 100000;

if ( simulation )
    
    basePath = sprintf('../data/simulated/explin%d/',nexp);
    alpha = load(strcat(basePath,'alpha'));
    CVfix = load(strcat(basePath,'cvfix'));
    startpose = load(strcat(basePath,'startpose'));
    robData = load(strcat(basePath,'dRob'));
    camData = loadDCam(strcat(basePath,'dCam'),1);
    [Map,Nfix] = LoadFMap(strcat(basePath,'fMap'),feetToMeters);
    truth = load(strcat(basePath,'groundTruth'));
    
    Nact = 1;
    N = Nfix + Nact;
    
    numIter=10;% number of iterations for IKF
    agg=5;% variance update frequency
    
    s = InitState(Map,N,startpose(1),startpose(2)); % Initial pose
    
else
    
    %%%%%%%%%%%%%%%
    % Settings and data sets for results presented in figures 8,9,10
    % " REAL ROBOT -- ESTIMATING THE MEASUREMENT NOISE COVARIANCE"
    %
    % alpha = [1e-5 1e-5 1e-5 1e-7  1e-7 1e-7]; %% TRUE ALPHA VALUES THIS LOOKS TO BE THE BEST
    % %alpha = [1e-5 1e-5 1e-5 1e-5  1e-7 1e-7];
    % %CVfix = 0.5;
    % CVfix = 0.538; %% TRUE  COVARIANCE FOR CALIBRATED DISTANCE SENSORS
    % %CVfix = 1.7119e-01;
    %
    % robData = load('../data/test2/dRob');
    % camData = loadDCam('../data/test2/dCam',inchesToMeters);
    % [Map,Nfix] = LoadFMap('../data/test1/fMap',feetToMeters);
    % Nact = 1;
    % N = Nfix + Nact;
    %
    % numIter=10;% number of iterations for IKF
    % agg=10;% variance update frequency
    %
    % s = InitState(Map,N,0,0,pi/2); % Initial pose
    %
    % trials = 1;
    
    %%%%%%%%%%%%%%
    
end

trials = 3;

fixed = s(1:2*Nfix);

x2 = s(2*Nfix+1:2*Nfix+2);
x3 = s(2*Nfix+1:2*Nfix+2);
x4 = s(2*Nfix+1:2*Nfix+2);

V2 = (1e-8)*eye(numel(x2),numel(x2));
V3 = (1e-8)*eye(numel(x3),numel(x3));
V4 = (1e-8)*eye(numel(x3),numel(x3));


alphatranfix = alpha;

mu = bias * alpha ;
sigma = 1e14*eye(numel(mu));
alphatranstim = mu;

%alphatranfix = c_error * alphatranfix;

T = size(robData,1);
ct = 1;

oldT = robData(1,1);
uw = zeros(N,1);
uv = zeros(N,1);

steps = size(robData,1);

steps = 300;

museries = zeros(size(mu,2),steps);
museries(:,1) = mu';

trajectory2 = zeros(steps,2); %% ESTIMATOR EKF1 (IKF)  (BLUE)
trajectory3 = zeros(steps,2); %% ESTIMATOR EST (ESTIMATOR PROPOSED IN THE PAPER) (GREEN)
trajectory4 = zeros(steps,2); %% ESTIMATOR EKF2 (EKF) (RED)

numSR = 0;

figure ;
hold on
grid on
plot(Map(:,2),Map(:,3),'r+'); % plot landmarks

if ( simulation )
    plot(truth(:,1),truth(:,2),'k'); % plot groundtruth
end



for t=1:steps
    
    dt = robData(t,1)-oldT;
    oldT = robData(t,1);
    
    if ( simulation )
        uv(N) = robData(t,5);
        uw(N) = robData(t,6);
    else
        uv(N) = robData(t,5)/1.2;
        uw(N) = (robData(t,6)-sign(uv(N))*dt*0.13*pi/180)/1.32;
    end
    
    [ValidLM,ct,measurements,c] = GetMeasurements(ct,camData,robData,Map,t,N);
    
    if ValidLM == false
        [x2,V2] = predict(x2,V2,uv(N),uw(N),dt,alphatranfix);
        [x3,V3] = predict(x3,V3,uv(N),uw(N),dt,alphatranstim);
        [x4,V4] = predict(x4,V4,uv(N),uw(N),dt,alphatranfix*c_error);
    else
        numSR = numSR + 1;
        
        %% Compute Q
        Q = ComputeQ(CVfix,measurements);
        [x2,V2,p] = iterate(x2,V2,numIter,uv(N),uw(N),measurements,c,dt,alphatranfix,Q,fixed);
        [x4,V4,p] = iterate(x4,V4,numIter,uv(N),uw(N),measurements,c,dt,alphatranfix*c_error,Q,fixed);
        
        %% ESTIMATED MODEL
        if  ( numSR < 51 )  || (  mod(numSR,agg) == 0  ) 
            if ( simulation )
                alphatranstimnew = EstimateCV(x3,V3,uv(N),uw(N),measurements,c,mu,sigma,dt,numIter,options,Q,fixed,trials);
                for k = 1:numel(alphatranstim)
                    if ( alphatranstimnew(k)> 5*alphatranstim(k) )
                        alphatranstim(k) = 5*alphatranstim(k);
                    elseif ( alphatranstimnew(k) < alphatranstim(k)/5 )
                        alphatranstim(k) = alphatranstim(k)/5;
                    else
                        alphatranstim(k) = alphatranstimnew(k);
                    end
                end
            else
                
                [alphatranstimnew,bestfval] = EstimateCV(x3,V3,uv(N),uw(N),measurements,c,mu,sigma,alpharotfix,dt,numIter,options,Q,fixed,trials);
                
                %  alphatranstim = bestalphatranstimnew;
                %             alphatranstimnew
                for k = 1:numel(alphatranstim)
                    if ( alphatranstimnew(k)> 10*alphatranstim(k) )
                        alphatranstim(k) = 10*alphatranstim(k);
                    elseif ( alphatranstimnew(k) < alphatranstim(k)/10 )
                        alphatranstim(k) = alphatranstim(k)/10;
                    else
                        alphatranstim(k) = alphatranstimnew(k);
                    end
                end
                aphatranstim = ( alphatranstimnew + alphatranstim ) / 2;
                %            alphatranstim = slidingaverage(museries,t,alphatranstimnew,wlen);
                %            if ( mod(numSR,50) == 0 )
                %                wlen = wlen+2;
                %            end
                % alphatranstim = alphatranstimnew;
            end
        end
        
        % alphatranstim
        
        [x3,V3,p] = iterate(x3,V3,numIter,uv(N),uw(N),measurements,c,dt,alphatranstim,Q,fixed);
        
        if   ( numSR < 51 )  || (  mod(numSR,agg) == 0  ) 
            [mu,sigma] = update_mu_sigma(alphatranstim,x3,V3,uv(N),uw(N),measurements,c,mu,sigma,dt,numIter,Q,fixed);
        end
        %else
        %    [x3,V3] = predict(x3,V3,uv(N),uw(N),dt,alphatranfix,alpharotfix);
        %end
        %      wlen
        
    end
    
    
    museries(:,t) = mu';
    
    %     trajectory0(t,1) = x0(1);
    %     trajectory0(t,2) = x0(2);
    %     trajectory0(t,3) = x0(3);
    %
    %     trajectory1(t,1) = x1(1);
    %     trajectory1(t,2) = x1(2);
    %     trajectory1(t,3) = x1(3);
    %
    %     trajectory2(t,1) = x2(1);
    %     trajectory2(t,2) = x2(2);
    %     trajectory2(t,3) = x2(3);
    %
    %     trajectory3(t,1) = x3(1);
    %     trajectory3(t,2) = x3(2);
    %     trajectory3(t,3) = x3(3);
    %
    %     trajectory4(t,1) = x4(1);
    %     trajectory4(t,2) = x4(2);
    %     trajectory4(t,3) = x4(3);
    
    
    trajectory2(t,:) = x2';
    trajectory3(t,:) = x3';
    trajectory4(t,:) = x4';
    
    plot(trajectory2(1:t,1),trajectory2(1:t,2),'b');
    plot(trajectory3(1:t,1),trajectory3(1:t,2),'g');
    plot(trajectory4(1:t,1),trajectory4(1:t,2),'r');
    pause(0.001);
    
end


% h = gca;
% set(h,'FontSize',14);
% if ( simulation )
%     print -depsc2 SimulationEvolutionPath
% else
%     print -depsc2 RobotEvolutionPath
% end

figure
semilogy(museries(1,:),'r')
hold on
semilogy(museries(2,:),'g')
% semilogy(museries(3,:),'b')
% semilogy(museries(4,:),'k')
ylabel('Estimated \theta');
legend('\theta_1','\theta_2');
grid on
h = gca;
set(h,'FontSize',18);
set(get(gca,'YLabel'),'FontSize',18)
set(get(gca,'XLabel'),'FontSize',18)
if ( simulation )
    s = sprintf('SimulationEvolutionAlphaExperiment%i',nexp);
    print( gcf, '-depsc2', s );
else
    s = sprintf('RealEvolutionAlphaExperiment%i',nexp);
    print( gcf, '-depsc2', s );
end

if ( simulation )
    
    figure
   % subplot(2,1,1)
    plot(sqrt((truth(1:steps,1)-trajectory3(:,1)).^2+(truth(1:steps,2)-trajectory3(:,2)).^2),'g');
    hold on
    plot(sqrt((truth(1:steps,1)-trajectory2(:,1)).^2+(truth(1:steps,2)-trajectory2(:,2)).^2),'b');
    plot(sqrt((trajectory4(:,1)-truth(1:steps,1)).^2+(trajectory4(:,2)-truth(1:steps,2)).^2),'r');
    grid on
    legend('EST','IKF1','IKF2');
    ylabel('Position error (m)');
    set(gca,'FontSize',18);
    set(get(gca,'XLabel'),'FontSize',18)
    set(get(gca,'YLabel'),'FontSize',18)
    
%     subplot(2,1,2)
%     plot(abs((truth(:,3)-trajectory3(:,3))*180/pi),'g');
%     hold on
%     plot(abs((truth(:,3)-trajectory2(:,3))*180/pi),'b');
%     plot(abs((truth(:,3)-trajectory4(:,3))*180/pi),'r');
%     legend('EST','EKF1','EKF2');
%     ylabel('Orientation error (degrees)');
%     grid on
%     set(gca,'FontSize',14);
%     set(get(gca,'XLabel'),'FontSize',14)
%     set(get(gca,'YLabel'),'FontSize',14)
%     
    s = sprintf('SimulationEvolutionErrorExperiment%i',nexp);
    print( gcf, '-depsc2', s );
    
else
    
    figure
    subplot(2,1,1)
    
    hold on
    plot(sqrt((trajectory3(:,1)-trajectory2(:,1)).^2+(trajectory3(:,2)-trajectory2(:,2)).^2),'g');
    plot(sqrt((trajectory4(:,1)-trajectory2(:,1)).^2+(trajectory4(:,2)-trajectory2(:,2)).^2),'r');
    grid on
    ylabel('Position error (m)');
    set(gca,'FontSize',14);
    set(get(gca,'XLabel'),'FontSize',14)
    set(get(gca,'YLabel'),'FontSize',14)
    
    subplot(2,1,2)
    hold on
    plot(abs((trajectory3(:,3)-trajectory2(:,3))*180/pi),'g');
    plot(abs((trajectory4(:,3)-trajectory2(:,3))*180/pi),'r');
    ylabel('Orientation error (degrees)');
    set(gca,'FontSize',14);
    set(get(gca,'XLabel'),'FontSize',14)
    set(get(gca,'YLabel'),'FontSize',14)
    grid on
    
    s = sprintf('RealEvolutionErrorExperiment%i',nexp);
    print( gcf, '-depsc2', s );
    
end


%
% figure
% subplot(2,1,1)
% plot(sqrt((truth(:,1)-trajectory3(:,1)).^2+(truth(:,2)-trajectory3(:,2)).^2),'g');
% hold on
% plot(sqrt((truth(:,1)-trajectory2(:,1)).^2+(truth(:,2)-trajectory2(:,2)).^2),'b');
% plot(sqrt((trajectory4(:,1)-truth(:,1)).^2+(trajectory4(:,2)-truth(:,2)).^2),'r');
% grid on
% % subplot(2,1,2)
% % plot(trajectory2(:,2)-trajectory3(:,2));
% % grid on
% legend('EST','EKF1','EKF2');
% ylabel('Position error (m)');
%
% subplot(2,1,2)
% plot((truth(:,3)-trajectory3(:,3))*180/pi,'g');
% hold on
% plot((truth(:,3)-trajectory2(:,3))*180/pi,'b');
% plot((truth(:,3)-trajectory4(:,3))*180/pi,'r');
% legend('EST','EKF1','EKF2');
% ylabel('Orientation error (degrees)');
% grid on




% figure
% subplot(2,1,1)
% plot(sqrt((trajectory2(:,1)-trajectory3(:,1)).^2+(trajectory2(:,2)-trajectory3(:,2)).^2));
% grid on
% % subplot(2,1,2)
% % plot(trajectory2(:,2)-trajectory3(:,2));
% % grid on
% subplot(2,1,2)
% plot(trajectory2(:,3)-trajectory3(:,3));
% grid on

% figure
% subplot(3,1,1)
% plot(trajectory2(:,1)-trajectory3(:,1));
% grid on
% subplot(3,1,2)
% plot(trajectory2(:,2)-trajectory3(:,2));
% grid on
% subplot(3,1,3)
% plot(trajectory2(:,3)-trajectory3(:,3));
% grid on

%     figure
%     subplot(2,1,1)
%
%     hold on
%     plot(sqrt((trajectory3(:,1)-trajectory2(:,1)).^2+(trajectory3(:,2)-trajectory2(:,2)).^2),'b');
%
%     grid on
%
%     ylabel('Position difference (m)');
%     subplot(2,1,2)
%     plot((trajectory3(:,3)-trajectory2(:,3))*180/pi,'b');
%     hold on
%     ylabel('Orientation difference (degrees)');
%     grid on
%
% figure
% semilogy(museries(1,:))
% hold on
% grid on
% semilogy(museries(2,:),'r')
% semilogy(museries(3,:),'g')
% semilogy(museries(4,:),'k')
%
% ylabel('Estimated \alpha');
% legend('\alpha_1','\alpha_2','\alpha_3','\alpha_4');
% grid on

fprintf('Average distance errors\n');
fprintf('EST: %f\n',mean((truth(1:steps,1)-trajectory3(:,1)).^2+(truth(1:steps,2)-trajectory3(:,2)).^2));
fprintf('IKF: %f\n',mean((truth(1:steps,1)-trajectory2(:,1)).^2+(truth(1:steps,2)-trajectory2(:,2)).^2));
fprintf('EKF: %f\n',mean((truth(1:steps,1)-trajectory4(:,1)).^2+(truth(1:steps,2)-trajectory4(:,2)).^2));

fprintf('Estimated value for unknown transition noise variances: %f\n',mean(museries(:,100:end),2));
fprintf('Real values for  sensor noise variances: %f\n',load(strcat(basePath,'alpha')));









