clear all
close all
format short e

% In order to replicate the experiments described in the paper you need to
% set only a few parameters: simulation, nexp, bias, c_error See
% documentation for details.

% true for simulations, false for experiments with the real robot

simulation = false;

% nexp: number of the experiment to be performed; it is needed to
% load the appropriate data files

nexp = 1;

% bias: positive multiplicative factor for the initial guess by the
% proposed  estimator;

bias = 1000;

% c_error: calibration error, i.e. the error in covarince values for
% EKF and IKF

c_error = 5;

%% Real processing starts here

% conversion factors
inchesToMeters = 0.0254;
feetToMeters = 0.3048;

options=optimset;
options.MaxIter=5000;

if ( simulation )
    
    basePath = sprintf('../data/simulated/exp%i/',nexp);
    alpha = load(strcat(basePath,'alpha'));
    CVfix = load(strcat(basePath,'cvfix'));
    startpose = load(strcat(basePath,'startpose'));
    robData = load(strcat(basePath,'dRob'));
    camData = loadDCam(strcat(basePath,'dCam'),1);
    [Map,Nfix] = LoadFMap(strcat(basePath,'fMap'),feetToMeters);
    truth = load(strcat(basePath,'groundTruth'));
    
    Nact = 1;
    N = Nfix + Nact;
    
    numIter=10;% number of iterations for IKF
    agg=5;% variance update frequency
    
    s = InitState(Map,N,startpose(1),startpose(2),startpose(3)); % Initial pose
    
else
    
    %%%%%%%%%%%%%%
    %Settings and data sets for results presented in figures 8,9,10
    %" REAL ROBOT -- ESTIMATING THE MEASUREMENT NOISE COVARIANCE"
    
    alpha = [1e-5 1e-5 1e-5 1e-7  1e-7 1e-7]; %% TRUE ALPHA VALUES THIS LOOKS TO BE THE BEST
    %alpha = [1e-5 1e-5 1e-5 1e-5  1e-7 1e-7];
    %CVfix = 0.5;
    CVfix = 0.538; %% TRUE  COVARIANCE FOR CALIBRATED DISTANCE SENSORS
    %CVfix = 1.7119e-01;
    
    basePath = sprintf('../data/test%i/',nexp);
    
    robData = load(strcat(basePath,'dRob'));
    camData = loadDCam(strcat(basePath,'dCam'),inchesToMeters);
    [Map,Nfix] = LoadFMap(strcat(basePath,'fMap'),feetToMeters);
    startpose = load(strcat(basePath,'startpose'));
    Nact = 1;
    N = Nfix + Nact;
    
    numIter=10;% number of iterations for IKF
    agg=1;% variance update frequency
    
    s = InitState(Map,N,startpose(1),startpose(2),startpose(3)); % Initial pose
    
    
    
end

trials =1;

x2 = s;
x3 = s;
x4 = s;

V2 = (1e-8)*eye(3*N,3*N);
V3 = (1e-8)*eye(3*N,3*N);
V4 = (1e-8)*eye(3*N,3*N);

mu = bias*CVfix;
sigma = 1e14*eye(1);

%CVfix = c_error * CVfix;

cvstim = mu;

T = size(robData,1);
ct = 1;

oldT = robData(1,1);
uw = zeros(N,1);
uv = zeros(N,1);

steps = size(robData,1);


museries = zeros(size(mu,1),steps);

museries(:,1) = mu;

% trajectory0 = zeros(steps,3); %% OPEN LOOP ESTIMATOR (PREDICTION ONLY) -- NOT USED IN THE PAPER
% trajectory1 = zeros(steps,3); %% IKF WITH JUST ONE ITERATION -- NOT USED IN THE PAPER
trajectory2 = zeros(steps,3); %% ESTIMATOR EKF1 (IKF WITH PERFECT PARAMETERS)  (BLUE)
trajectory3 = zeros(steps,3); %% ESTIMATOR EST (ESTIMATOR PROPOSED IN THE PAPER) (GREEN)
trajectory4 = zeros(steps,3); %% ESTIMATOR EKF2 (IKF WITH WRONG PARAMETERS) (RED)

numSR = 0;

figure ;
hold on
axis([-2 12 -2 13]);
grid on


if ( simulation )
    plot(truth(:,1),truth(:,2),'k'); % plot groundtruth
end

for t=1:steps
    
    dt = robData(t,1)-oldT;
    oldT = robData(t,1);
    
    if ( simulation )
        uv(N) = robData(t,5);
        uw(N) = robData(t,6);
    else
        uv(N) = robData(t,5)/1.2;
        uw(N) = (robData(t,6)-sign(uv(N))*dt*0.13*pi/180)/1.32;
    end
    
    
    %% OPEN LOOP PREDICTION
    %     [x0,V0] = predict(x0,V0,uv,uw,dt,alpha);
    
    [ValidLM,ct,measurements,c] = GetMeasurements(ct,camData,robData,Map,t,N);
    
    if ValidLM == false
        [x2,V2] = predict(x2,V2,uv,uw,dt,alpha);
        [x3,V3] = predict(x3,V3,uv,uw,dt,alpha);
        [x4,V4] = predict(x4,V4,uv,uw,dt,alpha);
    else
        numSR = numSR + 1;
        
        %% Compute Q
        Q = ComputeQ(CVfix,measurements);
        [x2,V2,p] = iterate(x2,V2,numIter,uv,uw,measurements,c,dt,alpha,Q);
        Q = ComputeQ(CVfix*c_error,measurements);
        [x4,V4,p] = iterate(x4,V4,numIter,uv,uw,measurements,c,dt,alpha,Q);
        
        %% ESTIMATED MODEL
        if  ( ( numSR > 0 ) && ( numSR < 51 ) ) || (  mod(numSR,agg) == 0  )
            if ( simulation )
                cvstim = EstimateCV(x3,V3,uv,uw,measurements,c,mu,sigma,dt,numIter,alpha,options,trials);
            else
                newcvstim = EstimateCV(x3,V3,uv,uw,measurements,c,mu,sigma,dt,numIter,alpha,options,trials);
                %    fprintf('Estimated %f\n',cvstim);
                if ( newcvstim > 10*cvstim )
                    newcvstim = 10*cvstim;
                elseif ( newcvstim < cvstim/10 )
                    newcvstim = cvstim/10;
                end
                cvstim = ( newcvstim + cvstim ) / 2;
            end
            %            cvstim = slidingaverage(museries,t,newcvstim,20);
            %             if ( mod(numSR,50) == 0 )
            %                 wlen = wlen+2;
            %             end
            
        end
        
        Q = ComputeQ(cvstim,measurements);
        [x3,V3,p] = iterate(x3,V3,numIter,uv,uw,measurements,c,dt,alpha,Q);
        
        if  ( ( numSR > 0 ) && ( numSR < 51 ) ) || (  mod(numSR,agg) == 0  )
            [mu,sigma] = update_mu_sigma(cvstim,x3,V3,uv,uw,measurements,c,mu,sigma,dt,numIter+3,alpha);
        end
        
    end
    
    museries(:,t) = mu;
    
    trajectory2(t,1) = x2(43);
    trajectory2(t,2) = x2(44);
    trajectory2(t,3) = x2(45);
    
    trajectory3(t,1) = x3(43);
    trajectory3(t,2) = x3(44);
    trajectory3(t,3) = x3(45);
    
    trajectory4(t,1) = x4(43);
    trajectory4(t,2) = x4(44);
    trajectory4(t,3) = x4(45);
    
    if  ( simulation )
        plot(trajectory2(1:t,1),trajectory2(1:t,2),'b');
        plot(trajectory3(1:t,1),trajectory3(1:t,2),'g');
        plot(trajectory4(1:t,1),trajectory4(1:t,2),'r');
    else
        plot(trajectory2(1:t,1),trajectory2(1:t,2),'b');
        plot(trajectory3(1:t,1),trajectory3(1:t,2),'g');
        
    end
    pause(0.001);
    
end

h = gca;
set(h,'FontSize',14);
if ( simulation )
    plot(Map(:,2),Map(:,3),'r+'); % plot landmarks
    s = sprintf('SimulationMeasurementPathExperiment%i',nexp);
    print( gcf, '-depsc2', s );
else
    legend('IKF1','EST');
    plot(Map(:,2),Map(:,3),'r+'); % plot landmarks
    s = sprintf('RobotMeasurementPathExperiment%i',nexp);
    print( gcf, '-depsc2', s );
end

figure
semilogy(museries)
ylabel('Estimated \zeta');
grid on
h = gca;
set(h,'FontSize',18);
set(get(gca,'YLabel'),'FontSize',18)
set(get(gca,'XLabel'),'FontSize',18)
if ( simulation )
    s = sprintf('SimulationMeasurementZetaExperiment%i',nexp);
    print( gcf, '-depsc2', s );
else
    s = sprintf('RealMeasurementZetaExperiment%i',nexp);
    print( gcf, '-depsc2', s );
end

if ( simulation )
    
    figure
    subplot(2,1,1)
    plot(sqrt((truth(:,1)-trajectory3(:,1)).^2+(truth(:,2)-trajectory3(:,2)).^2),'g');
    hold on
    plot(sqrt((truth(:,1)-trajectory2(:,1)).^2+(truth(:,2)-trajectory2(:,2)).^2),'b');
    plot(sqrt((trajectory4(:,1)-truth(:,1)).^2+(trajectory4(:,2)-truth(:,2)).^2),'r');
    grid on
    legend('EST','IKF1','IKF2');
    ylabel('Position error (m)');
    set(gca,'FontSize',14);
    set(get(gca,'XLabel'),'FontSize',18)
    set(get(gca,'YLabel'),'FontSize',18)
    
    subplot(2,1,2)
    plot(abs((truth(:,3)-trajectory3(:,3))*180/pi),'g');
    hold on
    plot(abs((truth(:,3)-trajectory2(:,3))*180/pi),'b');
    plot(abs((truth(:,3)-trajectory4(:,3))*180/pi),'r');
    legend('EST','IKF1','IKF2');
    ylabel('Orientation error (degrees)');
    grid on
    set(gca,'FontSize',14);
    set(get(gca,'XLabel'),'FontSize',18)
    set(get(gca,'YLabel'),'FontSize',18)
    
    s = sprintf('SimulationMeasurementErrorExperiment%i',nexp);
    print( gcf, '-depsc2', s );
    
else
    
    figure
    subplot(2,1,1)
    
    hold on
    plot(sqrt((trajectory3(:,1)-trajectory2(:,1)).^2+(trajectory3(:,2)-trajectory2(:,2)).^2),'g');
    %plot(sqrt((trajectory4(:,1)-trajectory2(:,1)).^2+(trajectory4(:,2)-trajectory2(:,2)).^2),'r');
    grid on
    ylabel('Position error (m)');
    set(gca,'FontSize',18);
    set(get(gca,'XLabel'),'FontSize',18)
    set(get(gca,'YLabel'),'FontSize',18)
    
    subplot(2,1,2)
    hold on
    plot(abs((trajectory3(:,3)-trajectory2(:,3))*180/pi),'g');
   % plot(abs((trajectory4(:,3)-trajectory2(:,3))*180/pi),'r');
    ylabel('Orientation error (degrees)');
    set(gca,'FontSize',18);
    set(get(gca,'XLabel'),'FontSize',18)
    set(get(gca,'YLabel'),'FontSize',18)
    grid on
    
    s = sprintf('RealMeasurementErrorExperiment%i',nexp);
    print( gcf, '-depsc2', s );
end
% 
% fprintf('Average distance errors\n');
% fprintf('EST: %f\n',mean((truth(:,1)-trajectory3(:,1)).^2+(truth(:,2)-trajectory3(:,2)).^2));
% fprintf('IKF: %f\n',mean((truth(:,1)-trajectory2(:,1)).^2+(truth(:,2)-trajectory2(:,2)).^2));
% fprintf('IKF2: %f\n',mean((truth(:,1)-trajectory4(:,1)).^2+(truth(:,2)-trajectory4(:,2)).^2));
% 
% fprintf('Estimated value for unknown sensor noise variance: %f\n',mean(museries(end-50:end)));
% fprintf('Real value for  sensor noise variance: %f\n',load(strcat(basePath,'cvfix')));

